﻿using AntiAircraftGun.Drawnings;
using AntiAircraftGun.Entities;

namespace AntiAircraftGun;
/// <summary>
/// Форма конфигурации объекта
/// </summary>
public partial class FormCarConfig : Form
{
    /// <summary>
	/// Объект - прорисовка бронемашины
	/// </summary>
    private DrawningArmoredCar? _armoredCar;
    /// <summary>
	/// Событие для передачи объекта
	/// </summary>
    private event Action<DrawningArmoredCar>? _armoredCarDelegate;
    public FormCarConfig()
    {
        InitializeComponent();
        panelRed.MouseDown += Panel_MouseDown;
        panelGreen.MouseDown += Panel_MouseDown;
        panelBlue.MouseDown += Panel_MouseDown;
        panelYellow.MouseDown += Panel_MouseDown;
        panelWhite.MouseDown += Panel_MouseDown;
        panelGray.MouseDown += Panel_MouseDown;
        panelBlack.MouseDown += Panel_MouseDown;
        panelPurple.MouseDown += Panel_MouseDown;

        buttonCancel.Click += (sender, e) => Close();

    }
    /// <summary>
	/// Привязка внешнего метода к событию
	/// </summary>
	/// <param name="carDelegate"></param>
	public void AddEvent(Action<DrawningArmoredCar> carDelegate)
    {
        _armoredCarDelegate += carDelegate;
    }
    /// <summary>
	/// Прорисовка объекта
	/// </summary>
	private void DrawObject()
    {
        Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _armoredCar?.SetPictureSize(pictureBoxObject.Width, pictureBoxObject.Height);
        _armoredCar?.SetPosition(15, 15);
        _armoredCar?.DrawTransport(gr);
        pictureBoxObject.Image = bmp;
    }
    /// <summary>
	/// Передаем информацию при нажатии на Label
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
    private void labelObject_MouseDown(object sender, MouseEventArgs e)
    {
        (sender as Label)?.DoDragDrop((sender as Label)?.Name ?? string.Empty, DragDropEffects.Move | DragDropEffects.Copy);
    }
    /// <summary>
    /// Проверка получаемой информации (ее типа на соответствие требуемому)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void panelObject_DragEnter(object sender, DragEventArgs e)
    {
        e.Effect = e.Data?.GetDataPresent(DataFormats.Text) ?? false ? DragDropEffects.Copy : DragDropEffects.None;
    }
    /// <summary>
	/// Действия при приеме перетаскиваемой информации
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
    private void panelObject_DragDrop(object sender, DragEventArgs e)
    {
        switch (e.Data?.GetData(DataFormats.Text)?.ToString())
        {
            case "labelSimpleObject":
                _armoredCar = new DrawningArmoredCar((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White);
                break;
            case "labelModifiedObject":
                _armoredCar = new DrawningAntiAircraftGun((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White,
                    Color.Black, checkBoxTower.Checked, checkBoxRadar.Checked);
                break;
        }

        DrawObject();
    }

    /// <summary>
	/// Передаем информацию при нажатии на Panel
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
	private void Panel_MouseDown(object? sender, MouseEventArgs e)
    {
        (sender as Control)?.DoDragDrop((sender as Control)?.BackColor, DragDropEffects.Move | DragDropEffects.Copy);
    }


    /// <summary>
    /// Передача основного цвета
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void labelBodyColor_DragEnter(object sender, DragEventArgs e)
    {
        if (e.Data.GetDataPresent(typeof(Color)))
        {
            e.Effect = DragDropEffects.Copy;
        }
        else
        {
            e.Effect = DragDropEffects.None;
        }
    }

    /// <summary>
    /// Прорисовка основным цветом
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void labelBodyColor_DragDrop(object sender, DragEventArgs e)
    {
        if (_armoredCar != null)
        {
            _armoredCar.EntityAircraftGun?.setBodyColor((Color)e.Data.GetData(typeof(Color)));
            DrawObject();
        }
    }
    /// <summary>
    /// Прорисовка основным цветом
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void labelAdditionalColor_DragDrop(object sender, DragEventArgs e)
    {
        if (_armoredCar?.EntityAircraftGun is EntityAntiAircraftGun _car)
        {
            _car.setAdditionalColor((Color)e.Data.GetData(typeof(Color)));
        }
        DrawObject();
    }
    /// <summary>
    /// Передача дополнительного цвета
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void labelAdditionalColor_DragEnter(object sender, DragEventArgs e)
    {
        if (_armoredCar is DrawningAntiAircraftGun)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
    }
    /// <summary>
	/// Передача объекта
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
    private void buttonAdd_Click_1(object sender, EventArgs e)
    {
        if (_armoredCar != null)
        {
            _armoredCarDelegate?.Invoke(_armoredCar);
            Close();
        }
    }
}
